import javax.swing.Timer;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Graphics2D;
import java.util.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.BasicStroke;
import java.awt.geom.Line2D;

public class MyWorld implements ActionListener {
	public static final double DEAFAUL_REFRESH = 0.003;  // 30 [ms]
	public static final double DEFAUL_DELTA = 0.0001;    // 1 [ms]
	public static int WIDTH = 600;  // in pixels
	public static int HEIGHT = 500; // in pixels
	public static int X_ORIGEN = (int)(WIDTH*0.1); // MyWorld space origin (x,y) will be on
	public static int Y_ORIGEN = (int)(HEIGHT*0.9); // (X_ORIGEN,Y_ORIGEN) of the panel space.
	public static AffineTransform SPACE_TRANSFORM;  // transforms (x,y) in (X,Y) of panel.
	public static AffineTransform SPACE_INVERSE_TRANSFORM; // transforms (X,Y) in (x,y) of my world.
	public static Line2D.Double SCALE_LINE;  // 1 [m] line
	private static double AXES_SCALE = 100.0;  // 1 [m] equals 100 [pixels]

	static {
		SPACE_TRANSFORM = AffineTransform.getTranslateInstance(X_ORIGEN, Y_ORIGEN);
		SPACE_TRANSFORM.scale(AXES_SCALE,-AXES_SCALE);  // it also inverts direction of y-coordinate 
		try {
			SPACE_INVERSE_TRANSFORM = SPACE_TRANSFORM.createInverse();
		}catch (NoninvertibleTransformException e){ 
			System.exit(0);
		}
		SCALE_LINE = new Line2D.Double(300,15, 300+AXES_SCALE,15);  // 1[m] line.
	}
	private double t;              // Simulation time
	private Timer pase;            // Clock to refresh the simulation's view. 
	private double delta_t;        // in seconds
	private double refreshPeriod;  // in seconds

	private Vector2D g;
	private float viscosity;
	private PhysicsLabPanel labPanel;
	private Vector<PhysicsElement> elements;

	public MyWorld(){
		t = 0;
		refreshPeriod = 0.04;      // initial value 40 [ms]
		delta_t = 0.001;          // initial value 1 [ms]
		pase = new Timer((int)(refreshPeriod*1000), this);
		g = new Vector2D(0,0);  // start with no gravity
		viscosity = 0.0f;        // no friction
		elements = new Vector<PhysicsElement>();
		labPanel = null;
	}
	public void setPanel (PhysicsLabPanel p) {
		labPanel = p;
	}
	public void setDelta_t(double delta) {
		delta_t = delta;
	}
	public void setRefreshPeriod (double rp) {
		refreshPeriod = rp;
		pase.setDelay((int)(refreshPeriod*1000)); // convert from [s] to [ms]
	}
	public void setGravity (Vector2D gravity) {
		g = gravity;
	}
	public Vector2D getGravity(){
		return g;
	}
	public void setViscosity(float v){
		viscosity = v;
	}
	public float getViscosity(){
		return viscosity;
	}

	public void addElement(PhysicsElement e) {
		elements.add(e);
		if (labPanel!=null)
			labPanel.repaint();
	}
	public void start() {
		if(pase.isRunning()) return;
		labPanel.desableMouseListener();
		pase.start();      
	}
	public void stop(){
		pase.stop();
		labPanel.enableMouseListener();
	}
	public void actionPerformed (ActionEvent event) {  // simulate time passing
		double nextStop=t+refreshPeriod;
		while (t<nextStop) {
			for (PhysicsElement e: elements)   // for each element
				e.computeNextState(delta_t, this);  // compute next state
			for (PhysicsElement e: elements)    // for all elements
				e.updateState();                // update its state
			t+=delta_t;
		}
		labPanel.repaint();		
	}
	public PhysicsElement getNearCollision (PhysicsElement anchor) { // first implementation
		for (PhysicsElement e: elements) 
			if ((anchor != e) && anchor.collide(e))
				return e;
		return null;      
	}
	public PhysicsElement getNearestCollision (PhysicsElement anchor) {
		double minDistance=Double.MAX_VALUE;
		double distance;
		PhysicsElement nearest=null;
		for (PhysicsElement e: elements) {
			if (anchor!=e) {
				distance = anchor.getVectorBetweenCenters(e).module()
				-anchor.getRadius()-e.getRadius();
				if ((distance< minDistance) && anchor.collide(e)){
					minDistance = distance;
					nearest = e;
				}
			}
		}
		return nearest;      
	}
	public void paintComponent(Graphics2D g2d){
		g2d.drawString("ELO329 1er.Sem. 2012,   1 [m] = ", 85, 15);
		g2d.draw(SCALE_LINE);
		g2d.transform(SPACE_TRANSFORM);
		g2d.setStroke(new BasicStroke(0.02f));
		for (PhysicsElement e: elements)
			if (e instanceof Drawable)
				((Drawable)e).draw(g2d);
	}
	public Drawable find(double d, double f) {
		for (PhysicsElement e: elements)
			if (e instanceof Drawable){
				Drawable de = (Drawable) e;
				if (de.contains(d,f)) return de;
			}
		return null;
	}  

} 
